package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.CabinetForm;
import digital.laboratory.platform.reagent.service.CabinetFormService;
import digital.laboratory.platform.reagent.vo.CabinetFormVO;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * @author Zhang Xiaolong created at  2023-03-22
 * @describe 前端控制器
 * <p>
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 * 为前端提供数据, 接受前端的数据
 * 为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 * 接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/cabinet_form")
@Api(value = "cabinet_form", tags = "柜子管理")
public class CabinetFormController {

    private final CabinetFormService cabinetFormService;

    /**
     * 通过id查询
     *
     * @param cabinetFormId id
     * @return R
     */
    @ApiOperation(value = "通过id查询柜子信息", notes = "通过id查询柜子信息")
    @GetMapping()
//	@PreAuthorize("@pms.hasPermission('reagent_cabinet_form_get')" )

    public R<CabinetFormVO> getById(String cabinetFormId, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        CabinetFormVO formById = cabinetFormService.getFormById(cabinetFormId);

        return R.ok(formById);

    }

    /**
     * 分页查询
     *
     * @param page        分页对象
     * @param cabinetForm
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page")
    @PreAuthorize("@pms.hasPermission('reagent_cabinet_form_get')")
    public R<IPage<CabinetForm>> getCabinetFormPage(Page<CabinetForm> page, CabinetForm cabinetForm, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        IPage<CabinetForm> cabinetFormSList = cabinetFormService.page(page, Wrappers.<CabinetForm>query()
                .orderByDesc("create_time")
        );
        return R.ok(cabinetFormSList);
    }


    /**
     * 新增
     *
     * @param cabinetForm
     * @return R
     */
    @ApiOperation(value = "新增柜子信息", notes = "新增柜子信息")
    @SysLog("新增柜子信息")
    @PostMapping
//    @PreAuthorize("@pms.hasPermission('reagent_cabinet_form_add')" )
    public R<CabinetForm> postAddObject(@RequestBody CabinetForm cabinetForm, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        CabinetForm form = cabinetFormService.addById(cabinetForm);

        if (form != null) {
            return R.ok(cabinetForm, "保存成功");
        } else {
            return R.failed(cabinetForm, "保存失败");
        }
    }

    /**
     * 修改
     *
     * @param cabinetForm
     * @return R
     */
    @ApiOperation(value = "修改柜子信息", notes = "新增柜子信息")
    @SysLog("新增柜子信息")
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_cabinet_form_edit')")
    public R<CabinetForm> putUpdateById(@RequestBody CabinetForm cabinetForm, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        CabinetForm form = cabinetFormService.editById(cabinetForm);

        if (form != null) {
            return R.ok(cabinetForm, "保存成功");
        } else {
            return R.failed(cabinetForm, "保存失败");
        }
    }

    /**
     * 通过id删除
     *
     * @param cabinetFormId id
     * @return R
     */
    @ApiOperation(value = "通过id删除柜子信息", notes = "通过id删除柜子信息")
    @SysLog("通过id删除柜子信息")
    @DeleteMapping("/{cabinetFormId}")
    @PreAuthorize("@pms.hasPermission('reagent_cabinet_form_del')")
    public R<String> deleteById(@PathVariable String cabinetFormId, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        if (cabinetFormService.delById(cabinetFormId)) {
            return R.ok("删除成功");
        } else {
            return R.failed("删除失败");
        }

    }

}
