package digital.laboratory.platform.reagent.controller;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.io.IoUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.plugin.table.LoopRowTableRenderPolicy;
import com.fasterxml.jackson.annotation.JsonFormat;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.feign.RemoteWord2PDFService;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.dto.PurchasingPlanDTO;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.service.PurchasingPlanService;
import digital.laboratory.platform.reagent.service.ReagentConsumablesService;
import digital.laboratory.platform.reagent.vo.ProcurementContentVO;
import digital.laboratory.platform.reagent.vo.PurchaseCatalogueVO;
import digital.laboratory.platform.reagent.vo.PurchasingPlanVO;
import digital.laboratory.platform.reagent.vo.ReplacementVO;
import feign.Response;
import lombok.Data;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.springframework.beans.factory.annotation.Autowired;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.security.Principal;
import java.sql.Date;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * (采购计划)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (采购计划) 前端控制器
 * <p>
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 * 为前端提供数据, 接受前端的数据
 * 为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 * 接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/purchasing_plan")
@Api(value = "purchasing_plan", tags = "采购计划管理")
public class PurchasingPlanController {

    @Autowired
    private final PurchasingPlanService purchasingPlanService;

    private final OssFile ossFile;

    private final RemoteWord2PDFService remoteWord2PDFService;

    private final ReagentConsumablesService reagentConsumablesService;

    /**
     * 通过id查询(采购计划)
     *
     * @param purchasingPlanId id
     * @return R
     */
    @ApiOperation(value = "点击一行通过id查询采购计划明细", notes = "点击一行通过id查询采购计划明细")
    @GetMapping()
    @PreAuthorize("@pms.hasPermission('reagent_purchasing_plan_get')")
    public R<PurchasingPlan> getById(String purchasingPlanId, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        PurchasingPlanVO purchasingPlanVO = purchasingPlanService.getPurchasingPlanVO(purchasingPlanId);
        return R.ok(purchasingPlanVO);
        //return R.ok(purchasingPlanService.getById(purchasingPlanId));
    }


    /**
     * 通过id查询(采购计划)
     *
     * @param id
     * @return R
     */
    @ApiOperation(value = "查询换货信息", notes = "查询换货信息")
    @GetMapping("/replacement")
    @PreAuthorize("@pms.hasPermission('reagent_purchasing_plan_replacemen')")
    public R<ReplacementVO> getReplacementVO(String id, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        ReplacementVO replacementVO = purchasingPlanService.getReplacementVO(id);

        return R.ok(replacementVO);
        //return R.ok(purchasingPlanService.getById(purchasingPlanId));
    }

    /**
     * 分页查询
     *
     * @param page 分页对象
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page")
    @PreAuthorize("@pms.hasPermission('reagent_purchasing_plan_get')")
    public R<IPage<PurchasingPlanVO>> getPurchasingPlanPage(Page<PurchasingPlan> page,  @DateTimeFormat(pattern = "yyyy-MM-dd") @JsonFormat(pattern = "yyyy-MM-dd") DateTime startTime,
                                                            @DateTimeFormat(pattern = "yyyy-MM-dd") @JsonFormat(pattern = "yyyy-MM-dd") DateTime endTime, HttpServletRequest theHttpServletRequest) {


        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        if (startTime != null && endTime != null) {

            IPage<PurchasingPlanVO> purchasingPlanVOPage = purchasingPlanService.getPurchasingPlanVOPage(page, Wrappers.<PurchasingPlan>query().
                    ge("create_time", startTime)
                    .le("create_time", endTime).orderByDesc("create_time"));

            return R.ok(purchasingPlanVOPage);
        }else {

        IPage<PurchasingPlanVO> purchasingPlanVOPage = purchasingPlanService.getPurchasingPlanVOPage(page, Wrappers.<PurchasingPlan>query()
                .orderByDesc("create_time"));

        return R.ok(purchasingPlanVOPage);
    }
}

    /**
     * 新增(采购计划)
     *
     * @param purchasingPlanDTOList (采购计划)
     * @return R
     */
    @ApiOperation(value = "新增(采购计划)", notes = "新增(采购计划) 参数DTO：（detailsOfCentralizedId，quantity = ，）")
    @SysLog("新增(采购计划)")
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_purchasing_plan_add')" )
    public R<PurchasingPlanVO> postAddObject(@RequestBody List<PurchasingPlanDTO> purchasingPlanDTOList, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        PurchasingPlanVO purchasingPlanVO = purchasingPlanService.addById(purchasingPlanDTOList, dlpUser);

        if (purchasingPlanVO != null) {
            return R.ok(purchasingPlanVO, "保存成功");
        } else {
            return R.failed("保存失败");

        }
    }

    /**
     * 修改采购计划明细
     *
     * @param purchasingPlanDTO (采购计划)
     * @return R
     */
    @ApiOperation(value = "修改采购计划明细", notes = "修改采购计划明细")
    @SysLog("修改(采购计划)")
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_procurement_content_edit')")
    public R<ProcurementContent> putUpdateById(@RequestBody PurchasingPlanDTO purchasingPlanDTO, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        ProcurementContent procurementContent = purchasingPlanService.editById(purchasingPlanDTO);

        if (procurementContent != null) {
            return R.ok(procurementContent, "修改成功");
        } else {
            return R.failed("修改失败");
        }
    }

    /**
     * 修改采购计划合并明细
     *
     * @param purchasingPlanDTO (采购计划DTO)
     * @return R
     */
    @ApiOperation(value = "修改采购计划合并明细", notes = "修改采购计划合并明细")
    @SysLog("修改(采购计划)")
    @PutMapping("/details")
    @PreAuthorize("@pms.hasPermission('reagent_procurement_content_merge')")
    public R<ProcurementContent> editDetails(@RequestBody PurchasingPlanDTO purchasingPlanDTO, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        ProcurementContent procurementContent = purchasingPlanService.editDetails(purchasingPlanDTO);

        if (procurementContent != null) {
            return R.ok(procurementContent, "修改成功");
        } else {
            return R.failed("修改失败");
        }
    }

    /**
     * 通过id删除采购计划明细
     *
     * @param procurementContentId id
     * @return R
     */
    @ApiOperation(value = "通过id删除采购计划明细", notes = "通过id删除采购计划明细")
    @SysLog("通过id删除采购计划明细")
    @DeleteMapping("/content")
    @PreAuthorize("@pms.hasPermission('reagent_procurement_content_del')")
    public R<String> deleteContentById(String procurementContentId, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        if (purchasingPlanService.delContentById(procurementContentId)) {
            return R.ok("移除成功");
        } else {
            return R.failed("移除失败");
        }

    }

    /**
     * 提交采购计划
     *
     * @param purchasingPlanId (采购计划)
     * @return R
     */
    @ApiOperation(value = "提交采购计划", notes = "提交采购计划")
    @SysLog("修改(采购计划)")
    @PutMapping("/commit")
    @PreAuthorize("@pms.hasPermission('reagent_purchasing_plan_commit')")
    public R<PurchasingPlan> commitById(String purchasingPlanId, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        PurchasingPlan purchasingPlan = purchasingPlanService.commitById(purchasingPlanId);

        if (purchasingPlan != null) {
            return R.ok(purchasingPlan, "提交成功");
        } else {
            return R.failed("提交失败");
        }
    }

    /**
     * 审核采购计划
     *
     * @param purchasingPlanDTO (采购计划)
     * @return R
     */
    @ApiOperation(value = "换货", notes = "换货")
    @SysLog("(换货)")
    @PutMapping("/replacement")
    @PreAuthorize("@pms.hasPermission('reagent_purchasing_plan_replacemen')")
    public R<ProcurementContent> replacementById(@RequestBody PurchasingPlanDTO purchasingPlanDTO, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        ProcurementContent procurementContent = purchasingPlanService.replacementById(purchasingPlanDTO);

        if (procurementContent != null) {
            return R.ok(procurementContent, "换货成功");
        } else {
            return R.failed("换货成功");
        }
    }

    /**
     * 审核采购计划
     *
     * @param auditAndApproveDto (采购计划)
     * @return R
     */
    @ApiOperation(value = "审核采购计划", notes = "审核采购计划")
    @SysLog("修改(采购计划)")
    @PutMapping("/audit")
    @PreAuthorize("@pms.hasPermission('reagent_purchasing_plan_audit')")
    public R<PurchasingPlan> auditById(@RequestBody AuditAndApproveDTO auditAndApproveDto, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        PurchasingPlan purchasingPlan = purchasingPlanService.auditById(auditAndApproveDto, dlpUser);

        if (purchasingPlan != null) {
            return R.ok(purchasingPlan, "审核成功");
        } else {
            return R.failed("审核失败");
        }
    }

    /**
     * 审批采购计划
     *
     * @param auditAndApproveDto (采购计划)
     * @return R
     */
    @ApiOperation(value = "审批采购计划", notes = "审批采购计划")
    @SysLog("修改(采购计划)")
    @PutMapping("/approve")
    @PreAuthorize("@pms.hasPermission('reagent_purchasing_plan_approve')")
    public R<PurchasingPlan> approveById(@RequestBody AuditAndApproveDTO auditAndApproveDto, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        PurchasingPlan purchasingPlan = purchasingPlanService.approveById(auditAndApproveDto, dlpUser);

        if (purchasingPlan != null) {
            return R.ok(purchasingPlan, "审批成功");
        } else {
            return R.failed("审批失败");
        }
    }
    /**-------采购计划打印*/
    /**
     * 采购计划打印
     *
     * @param purchasingPlanId
     *
     * @return
     */
    @ApiOperation(value = "采购计划打印", notes = "采购计划打印")
    @SysLog("采购计划打印")
    @PostMapping("/print")
	@PreAuthorize("@pms.hasPermission('reagent_purchasing_plan_print')")
    public void bizGetPDFInventory(String purchasingPlanId, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) {
        System.out.println("bizApplyWord.................");
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        PurchasingPlanVO purchasingPlanVO = purchasingPlanService.getPurchasingPlanVO(purchasingPlanId);

        List<ProcurementContentVO> voList = purchasingPlanVO.getProcurementContentVOList();

        if (voList.isEmpty()) {
            throw new RuntimeException("未查询到相关信息");
        }

        String id = purchasingPlanVO.getPurchasingPlanId();

        String applyFileName = "采购计划-"+id;

        String pdfFilePath = "document" + "/"+"purchasingPlan" + "/" + id + "/" + applyFileName + ".pdf";
        try {
            //直接调用pdf方法
            purchasingPlanTablePDF(purchasingPlanVO,id,theHttpServletRequest,httpServletResponse);
            ossFile.fileGet(pdfFilePath, httpServletResponse.getOutputStream());
            httpServletResponse.setContentType(applyFileName);
        } catch (Exception e) {
            System.out.println(String.format("minioFile objectExist() Exception. %s", e.getLocalizedMessage()));
            e.printStackTrace();
        }
    }


    private void purchasingPlanTablePDF(PurchasingPlanVO purchasingPlanVO, String id, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) throws Exception {
        System.out.println("PurchasingPlanTablePDF.................");

        List<ProcurementContentVO> procurementContentVOList = purchasingPlanVO.getProcurementContentVOList();

        ArrayList<ProcurementContentVO> voList = new ArrayList<>();

        for (ProcurementContentVO procurementContentVO : procurementContentVOList) {

            ReagentConsumables byId = reagentConsumablesService.getById(procurementContentVO.getReagentConsumableId());

            if (byId.getCategory().equals("试剂")|byId.getCategory().equals("耗材")){

                voList.add(procurementContentVO);
            }

        }

        //-----------------------------
        // 生成 word 版本的 采购目录

        String applyFileName = "采购计划-"+id;

        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        ossFile.fileGet("template" + "/" + "试剂耗材采购计划表模板.docx", bos);

        byte[] templateArray = bos.toByteArray();

        ByteArrayInputStream bis = new ByteArrayInputStream(templateArray);
        bos.close();

        LoopRowTableRenderPolicy policy = new LoopRowTableRenderPolicy();

        Configure config = Configure.builder().
                bind("voList", policy).build();
        /**
         * 循环表格
         */
        XWPFTemplate template = XWPFTemplate.compile(bis, config).render(
                new HashMap<String, Object>() {{
                    put("voList", voList);
                    put("appropriationBudget",purchasingPlanVO.getAppropriationBudget());
                    put("id",id);
                    put("size",1);
                    put("createName", purchasingPlanVO.getCreateName());
                    put("primaryAuditorName", purchasingPlanVO.getPrimaryAuditorName());
                    put("approverName", purchasingPlanVO.getApproverName());
                    put("createTime",LocalDateTimeUtil.format(purchasingPlanVO.getCommitTime(),"yyyy年MM月dd日"));
                    put("auditTimeOfPrimary",LocalDateTimeUtil.format(purchasingPlanVO.getAuditTimeOfPrimary(),"yyyy年MM月dd日"));
                    put("approvalTime",LocalDateTimeUtil.format(purchasingPlanVO.getApprovalTime(),"yyyy年MM月dd日"));
                }}
        );
        bis.close();

        ByteArrayOutputStream fosWord = new ByteArrayOutputStream();
        template.write(fosWord);
        template.close();

        //------------
        ByteArrayInputStream fisWord = new ByteArrayInputStream(fosWord.toByteArray());
        fosWord.close();

        //MockMultipartFile mockMultipartFile = new MockMultipartFile("file", entrustmentLetterFileName + ".docx", "application/vnd.openxmlformats-officedocument.wordprocessingml.document", fisWord);
        MockMultipartFile mockMultipartFile = new MockMultipartFile("file", applyFileName + ".docx", "image/jpg", fisWord);
        Response response = remoteWord2PDFService.word2pdf(mockMultipartFile);
        fisWord.close();


        ByteArrayOutputStream outPDF = new ByteArrayOutputStream();
        IoUtil.copy(response.body().asInputStream(), outPDF, IoUtil.DEFAULT_MIDDLE_BUFFER_SIZE);
        ByteArrayInputStream isPDF = new ByteArrayInputStream(outPDF.toByteArray());
        outPDF.close();


        ossFile.fileSave("document" + "/" + "purchasingPlan" + "/" + id + "/" + applyFileName + ".pdf", isPDF);
        isPDF.close();


        System.out.println(String.format("转换为 PDF 结束"));

    }



}
